/*
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.DataBean.DatabaseDataBean;

import java.sql.Connection;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import ca.uvic.csr.shrimp.DataBean.Artifact;
import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.DataBean.Relationship;

/**
 * Interacts specifically with a "SoftChange" database
 * 
 * @author Jie, Rob Lintern
 */
public class SoftChangeDataBean extends DatabaseDataBean {
    private Connection con;

    private Statement stmt;

    private ResultSet rs;

    public static final String AUTHOR_ART_TYPE = "AUTHOR";

    public static final String ATOMIC_CHANGE_ART_TYPE = "ATOMIC CHANGE";

    public static final String REVISIOIN_ART_TYPE = "FILE REVISIOIN";

    public static final String FILE_ART_TYPE = "FILE";

    public SoftChangeDataBean() {

        super("jdbc:postgresql://turingmachine.org/newevolution", "jiez", "rob", "org.postgresql.Driver");
        try {
            con = getConnection();
            stmt = con.createStatement();
        } catch (SQLException e) {
            System.err.println("Cannot connect to " + url);
            e.printStackTrace();
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#createArtifact(java.lang.Object)
     */
    protected Artifact createArtifact(Object artifactID) {
        // the id passed in should somehow uniquely identify some "artifact" in
        // the database,
        // perhaps by using a table name combined with a primary key?
        String nodeName = ((DatabaseArtifactID) artifactID).getPrimaryKey();
        String nodeType = ((DatabaseArtifactID) artifactID).getArtifactType();
        Artifact art = new DatabaseArtifact(this, nodeName, nodeType, artifactID);

        return art;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#createEmptyRelationship(java.lang.Object,
     *      java.lang.String, java.lang.String, java.util.Vector)
     */
    protected Relationship createEmptyRelationship(Object externalId, String name, String type, Vector artifacts) {
        Relationship rel = new DatabaseRelationship(this, name, type, artifacts, externalId);
        return rel;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#findOutgoingRelationshipsInBackEnd(ca.uvic.csr.shrimp.DataBean.Artifact)
     */
    protected Vector findOutgoingRelationshipsInBackEnd(Artifact srcArtifact) {
        Vector rels = new Vector();
        /*
         * TODO - must create relationships between artifacts here - an example
         * Object someParamatersThatIdentifyTheDestinationArtifact = new
         * Object(); DatabaseArtifactID artID =
         * createDatabaseArtifactID(someParamatersThatIdentifyTheDestinationArtifact);
         * if (artID != null) { Artifact destArtifact = findArtifact (artID); if
         * (destArtifact != null) { String relType = "someType"; Object relID =
         * correctID(null); Relationship rel = createRel(relID, srcArtifact,
         * destArtifact, relType); rels.add(rel); } }
         */
        return rels;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#findRootArtifactsInBackEnd(java.lang.String[])
     */
    protected Vector findRootArtifactsInBackEnd(String[] cprels) {
        // probably don't need to worry about this for now if calling from
        // CreoleDataBean
        return new Vector(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#findAllArtifactsInBackEnd()
     */
    protected Vector findAllArtifactsInBackEnd() {
        // gets every single artifact from the database, probably don't need to
        // worry about this for now
        // probably don't need to worry about this for now if calling from
        // CreoleDataBean
        return new Vector(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#findAllArtifactTypesInBackEnd()
     */
    protected Vector findArtifactTypesInBackEnd() {
        /*
         * TODO need to somehow get the artifact types from the database,
         * perhaps by reading a specific table, or just hard-coded.
         */
        return new Vector(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#findAllRelationshipTypesInBackEnd()
     */
    protected Vector findRelationshipTypesInBackEnd() {
        /*
         * TODO need to somehow get the relationship types from the database,
         * perhaps by reading a specific table, or just hard-coded.
         */
        return new Vector(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#addRelationshipToBackEnd(ca.uvic.csr.shrimp.DataBean.Relationship)
     */
    protected boolean addRelationshipToBackEnd(Relationship relationship) {
        // don't worry about this for now
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#removeRelationshipFromBackEnd(ca.uvic.csr.shrimp.DataBean.Relationship)
     */
    protected boolean removeRelationshipFromBackEnd(Relationship relationship) {
        // don't worry about this for now
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#getNominalAttributeNamesFromDomain()
     */
    protected String[] getNominalAttributeNamesFromDomain() {
        /*
         * TODO need to somehow get the nominal attribute names (ex. "author of
         * last commit") from the database, perhaps by reading a specific table,
         * or just hard-coded for now
         */
        return new String[] {};
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#getOrdinalAttributeNamesFromDomain()
     */
    protected String[] getOrdinalAttributeNamesFromDomain() {
        /*
         * TODO need to somehow get the ordinal attribute names (ex. "date of
         * last commit") from the database, perhaps by reading a specific table,
         * or just hard-coded for now
         */
        return new String[] {};
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#getNominalAttrValuesFromDomain(java.lang.String[],
     *      boolean)
     */
    protected Map getNominalAttrValuesFromDomain(boolean lookInBackEnd) {
        /*
         * TODO need to somehow get all the nominal attribute values (ex.
         * "author of last commit") from the database, usually by incrementally
         * storing attributes as new artifacts are created, but some other
         * method is possible, like using "SELECT DISTINCT" sql statement from a
         * specific table
         */
        return new HashMap();
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#getOrdinalAttrValuesFromDomain(java.lang.String[],
     *      boolean)
     */
    protected Map getOrdinalAttrValuesFromDomain(boolean lookInBackEnd) {
        /*
         * TODO need to somehow get all the ordinal attribute values (ex. "date
         * of last commit") from the database, usually by incrementally storing
         * attributes as new artifacts are created, but some other method is
         * possible, like using "SELECT DISTINCT" sql statement from a specific
         * table
         */
        return new HashMap();
    }

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.DataBean#getArtifactIDFromString(java.lang.String)
     */
    public Object getArtifactExternalIDFromString(String idInStringForm) {
        // probably don't need to worry about this for now if calling from
        // CreoleDataBean
        // TODO need to implement this if you want snapshots, or any other
        // persistent storage to work
        return idInStringForm;
    }
    
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.DataBean#getStringFromExternalArtifactID(java.lang.Object)
     */
    public String getStringFromExternalArtifactID(Object externalID) {
        return externalID.toString();
    }
    
    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.DataBean#getStringFromExternalRelationshipID(java.lang.Object)
     */
    public String getStringFromExternalRelationshipID(Object externalID) {
        return externalID.toString();
    }
    
    

    /*
     * (non-Javadoc)
     * 
     * @see ca.uvic.csr.shrimp.DataBean.DataBean#getDefaultCprels()
     */
    public String[] getDefaultCprels() {
        // probably don't need to worry about this for now if calling from
        // CreoleDataBean
        return new String[] {};
    }
    
    public boolean getDefaultCprelsInverted() {
        return false;
    }

    /*
     * private DatabaseArtifactID createDatabaseArtifactID (String uniqueID) {
     * DatabaseArtifactID id = new DatabaseArtifactID (uniqueID); return id; }
     */

    /**
     * @param artifact
     */
    public void setCVSAttributes(Artifact artifact) {
        //    			IResource resource = (((CreoleArtifact)artifact).getResource()) ;
        //    			String fileName = ((resource.getFullPath()).toString()).substring(1);
        //    			//System.out.println(fileName);
        //				String authorWithMostEntries = findAuthorWithMostChanges (fileName);
        //				if (!authorWithMostEntries.equals("")) {
        //					artifact.setAttribute(SoftwareDomainConstants.ATTR_CVS_AUTHOR_MOST_COMMITS,
        // authorWithMostEntries);
        //				}
        //				artifact.setAttribute(SoftwareDomainConstants.ATTR_CVS_NUM_COMMITS,
        // new Integer(findNumberOfCommits(fileName)));
        //				
        //				artifact.setAttribute(SoftwareDomainConstants.ATTR_CVS_COMMIT_DATE_OLDEST,
        // findOldestCommitDate(fileName));
        //				
        //				artifact.setAttribute(SoftwareDomainConstants.ATTR_CVS_COMMIT_DATE_NEWEST,
        // findNewestCommitDate(fileName));
        //				
        //				artifact.setAttribute(SoftwareDomainConstants.ATTR_CVS_AUTHOR_FIRST,
        // findFirstAuthor(fileName));
        //				
        //				artifact.setAttribute(SoftwareDomainConstants.ATTR_CVS_AUTHOR_LAST,
        // findLastAuthor(fileName));

    }

    /**
     * @param fileName
     */
    private String findLastAuthor(String fileName) {
        String authorLast = null;
        String SQL_LAST_AUTHOR = "select author " + "from mrs2 " + "where daterev = (select max(daterev) as maxdate " + "from mrs2 " + "where mrid in (select mrid " + "from mrsfiles " + "where filename = '" + fileName + "')) " + "and mrid in (select mrid " + "from mrsfiles " + "where filename = '"
                + fileName + "') ";
        System.out.println(SQL_LAST_AUTHOR);
        try {
            rs = stmt.executeQuery(SQL_LAST_AUTHOR);
            if (rs.next()) {
                authorLast = rs.getString(1);
            }
        } catch (SQLException e) {
            System.out.println("Failed to find last author of file:" + fileName);
            e.printStackTrace();
        }
        return authorLast;
    }

    /**
     * @param fileName
     */
    protected String findFirstAuthor(String fileName) {
        String authorFirst = null;
        String SQL_FIRST_AUTHOR = "select author " + "from mrs2 " + "where daterev = (select min(daterev) as mindate " + "from mrs2 " + "where mrid in (select mrid " + "from mrsfiles " + "where filename = '" + fileName + "')) " + "and mrid in (select mrid " + "from mrsfiles " + "where filename = '"
                + fileName + "') ";
        try {
            rs = stmt.executeQuery(SQL_FIRST_AUTHOR);
            if (rs.next()) {
                authorFirst = rs.getString(1);
            }
        } catch (SQLException e) {
            System.out.println("Failed to find first author of file:" + fileName);
            e.printStackTrace();
        }
        return authorFirst;
    }

    /**
     * @param fileName
     */
    protected Date findNewestCommitDate(String fileName) {
        Date newestCommitDate = null;
        String SQL_NEWEST_COMMIT_DATE = "select max(daterev) as maxdate " + "from mrs2 " + "where mrid in (select mrid " + "from mrsfiles " + "where filename = '" + fileName + "') ";
        try {
            rs = stmt.executeQuery(SQL_NEWEST_COMMIT_DATE);
            if (rs.next()) {
                newestCommitDate = rs.getDate(1);
            }
        } catch (SQLException e) {
            System.out.println("Failed to find newest commit date of the file:" + fileName);
            e.printStackTrace();
        }
        return newestCommitDate;
    }

    /**
     * @param fileName
     */
    protected Date findOldestCommitDate(String fileName) {
        Date oldestCommitDate = null;
        String SQL_OLDEST_COMMIT_DATE = "select min(daterev) as mindate " + "from mrs2 " + "where mrid in (select mrid " + "from mrsfiles " + "where filename = '" + fileName + "') ";
        try {
            rs = stmt.executeQuery(SQL_OLDEST_COMMIT_DATE);
            if (rs.next()) {
                oldestCommitDate = rs.getDate(1);
            }
        } catch (SQLException e) {
            System.out.println("Failed to find oldest commit date of the file:" + fileName);
            e.printStackTrace();
        }
        return oldestCommitDate;
    }

    /**
     * @param fileName
     */
    protected int findNumberOfCommits(String fileName) {
        int numberOfCommits = 1;
        String SQL_NUMBER_COMMITS = "select count(*) as count " + "from mrs2 " + "where mrid in (select mrid " + "from mrsfiles " + "where filename = '" + fileName + "') ";
        try {
            rs = stmt.executeQuery(SQL_NUMBER_COMMITS);
            if (rs.next()) {
                numberOfCommits = rs.getInt(1);
            }
        } catch (SQLException e) {
            System.out.println("Failed to find number of commits of the file:" + fileName);
            e.printStackTrace();
        }
        return numberOfCommits;

    }

    /**
     * @param fileName
     */
    private String findAuthorWithMostChanges(String fileName) {
        String authorWithMostChanges = null;
        String SQL_AUTHORS_WITH_CHANGES = "select author " + "from mrs2 " + "where mrid in (select mrid " + "from mrsfiles " + "where filename = '" + fileName + "') " + "order by author";
        try {
            rs = stmt.executeQuery(SQL_AUTHORS_WITH_CHANGES);

            int author1_count = 0;
            int author2_count = 0;
            String author1 = null;
            String author2 = null;
            if (rs.next()) {
                authorWithMostChanges = author1 = rs.getString(1);
                author1_count++;
            }
            while (rs.next()) {
                String tempAuthor = rs.getString(1);
                if (tempAuthor.equals(author1)) {
                    author1_count++;
                } else {
                    if (author2 == null) {
                        author2 = tempAuthor;
                        author2_count++;
                    } else if (tempAuthor.equals(author2)) {
                        author2_count++;
                    } else {
                        if (author1_count < author2_count) {
                            authorWithMostChanges = author1 = author2;
                            author1_count = author2_count;
                        }
                        author2 = null;
                        author2_count = 0;
                    }
                }
            }
        } catch (SQLException e) {
            System.out.println("Failed to find author with most changes of the file:" + fileName);
            e.printStackTrace();
        }
        return authorWithMostChanges;
    }


    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#findIncomingRelationshipsInBackEnd(ca.uvic.csr.shrimp.DataBean.Artifact)
     */
    protected Vector findIncomingRelationshipsInBackEnd(Artifact artifact) {
        // TODO implement SoftChangeDataBean.findIncomingRelationshipsInBackEnd
        return new Vector(0);
    }

    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.DataBean.AbstractDataBean#addArtifactToBackEnd(ca.uvic.csr.shrimp.DataBean.Artifact)
     */
    protected boolean addArtifactToBackEnd(Artifact artifact) {
        // TODO implement SoftChangeDataBean.addArtifactToBackEnd
        return false;
    }
    
    /** For testing * */
    public static void main(String[] argv) {
        DataBean dataBean = new SoftChangeDataBean();
        System.out.println(((SoftChangeDataBean) dataBean).findLastAuthor("calendar/ChangeLog"));

        System.out.println(((SoftChangeDataBean) dataBean).findAuthorWithMostChanges("calendar/ChangeLog"));
        //Vector allArtifacts = dataBean.getArtifacts(true);
        //System.out.println("allArtifacts.size(): " + allArtifacts.size());

        //Vector allRels = dataBean.getRelationships(true);
        //System.out.println("allRels.size(): " + allRels.size());

        //Vector artTypes = dataBean.getArtifactTypes(true, true);
        //System.out.println("artTypes: " + artTypes);

        //Vector relTypes = dataBean.getRelationshipTypes(true, true);
        //System.out.println("relTypes: " + relTypes);

    }


}
