/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;

/**
 * @author Rob Lintern/Casey Best
 */
public class NewNodeDialog extends JDialog {
	private JTextField txtName;
	private JComboBox cmbxType;
	private JComboBox cmbxCprel;
	
	private boolean accepted = false;
	
	public NewNodeDialog (Object [] types, boolean allowNewType) {
	    this (types, new String [] {}, allowNewType);
	}
	
	public NewNodeDialog (Object [] types, String [] cprels, boolean allowNewType) {
		super (ApplicationAccessor.getApplication().getParentFrame(), "New Node Creator", true);
		
		JPanel fieldPanel = new JPanel();
		fieldPanel.setLayout(new GridLayout(0,4));
		
		JLabel nameLabel = new JLabel ("Name: ");
		fieldPanel.add (nameLabel);
		txtName = new JTextField (12);
		fieldPanel.add (txtName);
		
		JLabel typeLabel = new JLabel ("Type: ");
		fieldPanel.add (typeLabel);
		cmbxType = new JComboBox (types);
		cmbxType.setEditable(allowNewType);
		cmbxType.setSelectedIndex (0);
		fieldPanel.add (cmbxType);
		
		if (cprels.length > 1) {
			JLabel cprelLabel = new JLabel ("Child-Parent Relationship: ");
			fieldPanel.add (cprelLabel);
			cmbxCprel = new JComboBox (cprels);
			cmbxCprel.setEditable(allowNewType);
			cmbxCprel.setSelectedIndex (0);
			fieldPanel.add (cmbxCprel);
		}

		JPanel buttonPanel = new JPanel();
		JButton okButton = new JButton ("Ok");
		okButton.addActionListener (new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				accepted = true;					
				dispose();
			}
		});
		buttonPanel.add (okButton, BorderLayout.WEST);
		JButton cancelButton = new JButton ("Cancel");
		cancelButton.addActionListener (new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				accepted = false;
				dispose();
			}
		});
		buttonPanel.add (cancelButton, BorderLayout.EAST);

		getContentPane().add (fieldPanel, BorderLayout.CENTER);
		getContentPane().add (buttonPanel, BorderLayout.SOUTH);
		pack();
		// relocate and setVisible
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension screen = toolkit.getScreenSize();
		int x = (int)(screen.getWidth() - getWidth()) / 2;
		int y = (int)(screen.getHeight() - getHeight()) / 2;
		setLocation (x,y);
		setVisible (true);
	}
	/** 
	 * Returns whether or not the user has cancelled this dialog box.
	 */
	public boolean accepted () {
		return accepted;
	}
	
	/** 
	 * Returns the name for the new node, as entered by the user.
	 */
	public String getName() {
		return txtName.getText();
	}
	
	/**
	 * Returns the type for the new node, as chosen by the user
	 */
	public String getType() {
		return cmbxType.getSelectedItem().toString();
	}
	
	/**
	 * Returns the cprel for the new node, as chosen by the user
	 */
	public String getCprel() {
		return cmbxCprel.getSelectedItem().toString();
	}

}
